/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Kevin Teuscher
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// note: global vars in this file don't need "prefbar" prefix in their names

var gActiveTree = null;
var allTree = null;
var enabledTree = null;

var gRDF = null;
var gMainDS = null;

function Startup() {
  setTimeout(DelayedStartup, 0);
}
function DelayedStartup() {
  allTree = document.getElementById("allTree");
  enabledTree = document.getElementById("enabledTree");

  allTree.addEventListener("click", TreeClick, true);
  enabledTree.addEventListener("click", TreeClick, true);

  gRDF = goPrefBar.RDF;
  gMainDS = gRDF.mDatasource;
  gRDF.AddmDatasource(allTree);
  gRDF.AddmDatasource(enabledTree);

  // Set the preset for the selections.
  enabledTree.focus();
  if (allTree.view) allTree.view.selection.clearSelection();
  if (enabledTree.view) enabledTree.view.selection.select(0);

  window.addEventListener("unload", prefbarWriteToDisk, true);
}

function TreeClick(event) {
  if (event.detail != 2) return; // doubleclick

  event.preventDefault();
  event.stopPropagation();
  prefbarItemEdit();
}

function TreeKeypress(event) {
  if (event.keyCode != 46) return;

  prefbarItemDelete();
}

function AllTreeFocus() {
  // Remove selection from other tree
  if (enabledTree.view) enabledTree.view.selection.clearSelection();
  gActiveTree = allTree;
}
function EnabledTreeFocus() {
  // Remove selection from other tree
  if(allTree.view) allTree.view.selection.clearSelection();
  gActiveTree = enabledTree;
}

function PopupShowing() {
  if (!gActiveTree) return false;

  var selectionCount = 0;
  if (gActiveTree.view) selectionCount = gActiveTree.view.selection.count;

  var editButton = document.getElementById("itemEdit");
  var copyButton = document.getElementById("itemCopy");
  var deleteButton = document.getElementById("itemDelete");
  var exportButton = document.getElementById("itemExport");

  editButton.setAttribute("disabled", (selectionCount != 1));
  copyButton.setAttribute("disabled", (selectionCount != 1));
  deleteButton.setAttribute("disabled", (selectionCount <= 0));
  exportButton.setAttribute("disabled", (selectionCount <= 0));
  return true;
}


function ItemNew(aType) {
  if (!aType) return;
  var item1 = enabledTree.contentView.getItemAtIndex(0);
  window.openDialog("chrome://prefbar/content/newItem/newItem.xul", "newItemDialog", "chrome,titlebar,dialog,modal,resizable", aType);
  var item2 = enabledTree.contentView.getItemAtIndex(0);

  if (item1 != item2) {
    enabledTree.builder.rebuild();
    enabledTree.focus();
    setTimeout(function() {
      enabledTree.view.selection.select(0);
    });
  }
}

function prefbarItemEdit() {
  var selections = prefbarGetTreeSelections(gActiveTree);
  if (selections.length != 1) return;
  var selItemId = selections[0];

  window.openDialog("chrome://prefbar/content/newItem/newItem.xul", "editItemDialog", "chrome,titlebar,dialog,modal,resizable", selItemId);
}


function prefbarItemDelete() {
  var selections = prefbarGetTreeSelections(gActiveTree);
  var len = selections.length;
  if (len < 1) return;

  if(!goPrefBar.msgYesNo(window, goPrefBar.GetString("pref-editbar.properties", "questiondelete"))) return;

  var items = {};
  for(var i = 0; i < len; i++) {
    var selItemId = selections[i];
    var selItemCN = getCNameById(selItemId);
    items[selItemId] = selItemCN;
  }

  gMainDS.beginUpdateBatch();

  for (var delitem in items) {
    var parent = items[delitem];

    // If the item, to delete, is a container, then move all children to
    // the parent container, to empty the container.
    if (gRDF.IsContainer(gMainDS, delitem)) {
      var index = gRDF.GetIndexOf(gMainDS, parent, delitem);

      var children = gRDF.GetChildNodes(gMainDS, delitem);
      gRDF.RemoveChildNodes(gMainDS, delitem, children);
      gRDF.AddChildNodes(gMainDS, parent, children, index);

      for (var index in children) {
        var child = children[index];
        if (child in items) items[child] = parent;
      }
    }

    // Drop all attributes and pseudo-attributes from item
    var attributes = gRDF.GetAttributes(gMainDS, delitem, true);
    gRDF.RemoveAttributes(gMainDS, delitem, attributes);

    // Drop item from its parent
    gRDF.RemoveChildNodes(gMainDS, parent, delitem);
  }

  gMainDS.endUpdateBatch();
}

function ItemCopy() {
  var selections = prefbarGetTreeSelections(gActiveTree);
  if (selections.length != 1) return;
  var selItemId = selections[0];
  if (!selItemId.match(/^(urn:prefbar:(?:browser)?buttons:)/)) return;
  var prefix = RegExp.$1;

  var copyid;
  var id = "";
  while(true) {
    id = goPrefBar.msgPrompt(window, goPrefBar.GetString("pref-editbar.properties", "promptenterid"), id);
    if (id === null) return;
    if (!id.match(/[^\n\t\r ]/)) continue;
    copyid = prefix + id;
    if (gRDF.NodeExists(gMainDS, copyid)) {
      goPrefBar.msgAlert(window, goPrefBar.GetString("newItem.properties", "alertidinuse"));
    }
    else break;
  }

  gMainDS.beginUpdateBatch();

  // Source is container --> Make destination a new container
  if (gRDF.IsContainer(gMainDS, selItemId))
    gRDF.MakeContainer(gMainDS, copyid);

  // Copy over attributes
  var attributes = gRDF.GetAttributes(gMainDS, selItemId);
  for (var attrindex in attributes) {
    var attrib = attributes[attrindex];
    var value = gRDF.GetAttributeValue(gMainDS, selItemId, attrib);
    gRDF.SetAttributeValue(gMainDS, copyid, attrib, value);
  }

  // Add new button to the container node of the active tree
  var container = gActiveTree.ref;
  gRDF.AddChildNodes(gMainDS, container, copyid);

  gMainDS.endUpdateBatch();

  setTimeout(function() {
    gActiveTree.view.selection.select(0);
  });
}

function prefbarItemExport() {
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"]
                     .createInstance(nsIFilePicker);

  fp.init(window, "Export Buttons to File:", nsIFilePicker.modeSave);
  fp.appendFilter("ButtonCollections", "*.btn");
  fp.defaultExtension = "btn";

  var res=fp.show();
  if (res==nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace) {
    var selections = prefbarGetTreeSelections(gActiveTree);
    goPrefBar.ImpExp.Export(window, selections, fp.fileURL.spec);
  }
}


function prefbarItemImport() {
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"]
                     .createInstance(nsIFilePicker);

  fp.init(window, "Import Buttons from File:", nsIFilePicker.modeOpen);
  fp.appendFilter("ButtonCollections", "*.btn");
  fp.defaultExtension = "btn";

  var res=fp.show();
  if (res==nsIFilePicker.returnOK) {
    goPrefBar.ImpExp.Import(window, fp.fileURL.spec);
  }
}

const gDragFlavor = "text/x-moz-prefbar-button";
var gDragArray;

function BeginDragTree(event, tree) {
  goPrefBar.dump("BeginDragTree");

  if (event.originalTarget.localName != "treechildren")
    return false;

  var row = tree.treeBoxObject.getRowAt(event.clientX, event.clientY);
  goPrefBar.dump("BeginDragTree: row: " + row);
  if (row == -1)
    return false;

  gDragArray = new Array();

  var selection = prefbarGetTreeSelections(tree);
  for (var i = 0; i < selection.length; ++i ) {
    var curItem = selection[i];
    var ItemArray = new Array(curItem, getCNameById(curItem));
    gDragArray.push(ItemArray);
  }

  // We just drag the word "prefbar" the usual way.
  // The real data is in gDragArray.
  event.dataTransfer.effectAllowed = "move";
  event.dataTransfer.setData(gDragFlavor, "prefbar");

  return false;  // don't propagate the event if a drag has begun
}

function DragOverTree(event) {
  if (event.dataTransfer.types.contains(gDragFlavor)) {
    event.preventDefault();
    return false;
  }

  return true;
}

function DropOnTree(event, tree) {
  goPrefBar.dump("DropOnTree");

  // The drop procedure may take really long (depending on how much items
  // have to be moved)...
  window.setCursor("wait");
  saveOpenStates();

  var idPlaceBefore = null;
  var idPlaceInside = null;
  var PlaceAfter = false;

  var tbo = tree.treeBoxObject;
  if (tbo.treeBody && tbo.treeBody.childNodes.length > 0) {
    var row = tbo.getRowAt(event.clientX, event.clientY);
    goPrefBar.dump("DropOnTree: Dropped On Row: " + row);
    if (row != -1) {
      var rowHeight = tbo.rowHeight;
      var rowY = tbo.treeBody.boxObject.y;
      var firstRow = tbo.getFirstVisibleRow();

      var ItemPos = rowY + rowHeight * (row - firstRow);
      var pastePos = (event.clientY - ItemPos) / (rowHeight / 4);

      idPlaceBefore = tree.contentView.getItemAtIndex(row).getAttribute("id");

      var item = document.getElementById(idPlaceBefore);
      if (item.getAttribute("container") == "true") {
        if (item.getAttribute("empty") == "true") {
          if (pastePos > 1 && pastePos < 3) {
            idPlaceInside = idPlaceBefore;
            idPlaceBefore = null;
          }
          else if (pastePos >= 3)
            PlaceAfter = true;
        }
        else if (item.getAttribute("open") == "true") {
          if (pastePos > 2) {
            idPlaceInside = idPlaceBefore;
            idPlaceBefore = null;
          }
        }
        else if (pastePos > 2)
          PlaceAfter = true;
      }
      else if (pastePos > 2)
        PlaceAfter = true;
    }
    else {
      goPrefBar.dump("DropOnTree: Dropped after last item");
      var lastRow = tbo.treeBody.childNodes.length - 1;
      idPlaceBefore = tbo.treeBody.childNodes[lastRow].getAttribute("id");
      PlaceAfter = true;
    }
  }
  else {
    goPrefBar.dump("DropOnTree: Dropped On Empty Tree");
  }

  // Get target sequence and container
  var targetcontainer;
  if (idPlaceBefore)
    targetcontainer = getCNameById(idPlaceBefore);
  else if (idPlaceInside) {
    targetcontainer = idPlaceInside;
    savedOpenState[idPlaceInside] = "true";
  }
  else
    targetcontainer = tree.ref;

  // Containers may only be moved as one single item and they may only be
  // moved to the enabled root level. Otherwise delete all containers
  // from gDragArray
  if (gDragArray.length > 1 ||
      targetcontainer != "urn:prefbar:browserbuttons:enabled") {
    for (var i = gDragArray.length - 1; i >= 0; i--) {
      if (gRDF.IsContainer(gMainDS, gDragArray[i][0]))
        gDragArray.splice(i, 1);
    }
  }

  gMainDS.beginUpdateBatch();

  // first run: Kill all items from old seq
  var dragcnt = gDragArray.length;
  for (var i = 0; i < dragcnt; i++) {
    var curitem = gDragArray[i];
    gRDF.RemoveChildNodes(gMainDS, curitem[1], curitem[0]);
  }

  // Now get target position.
  var newpos;
  if (!idPlaceBefore)
    newpos = 1;
  else {
    newpos = gRDF.GetIndexOf(gMainDS, targetcontainer, idPlaceBefore);
    if (newpos == -1)
      newpos = 1;
    else
      if (PlaceAfter) newpos++;
  }

  // Second run: Add all items to new seq on their new position
  for (var i = 0; i < dragcnt; i++) {
    var curitem = gDragArray[i];
    gRDF.AddChildNodes(gMainDS, targetcontainer, curitem[0], newpos);
    newpos++;
  }

  gMainDS.endUpdateBatch();

  // Focus target tree
  tree.builder.rebuild();
  tree.focus();
  restoreOpenStates();

  // Add selection for newly added items
  if (dragcnt > 0) {
    for (var index = 0; index < tree.view.rowCount; index++) {
      if (tree.contentView.getItemAtIndex(index).id == gDragArray[0][0]) {
        tree.view.selection.rangedSelect(index, index + gDragArray.length - 1, false);
        break;
      }
    }
  }

  window.setCursor("auto");
  return true;
}


function getCNameById(id) {
  var item = document.getElementById(id);
  item = item.parentNode.parentNode;
  if (item.localName == "treeitem")
    return item.id;
  else
    return item.ref;
}

var savedOpenState;
function saveOpenStates() {
  savedOpenState = {};
  var tb = enabledTree.treeBoxObject.treeBody;
  if (!tb) return; // Empty tree
  var len = tb.childNodes.length;
  for (var index = 0; index < len; index++) {
    var curnode = tb.childNodes[index];
    if (curnode.getAttribute("container") == "true")
      savedOpenState[curnode.id] = curnode.getAttribute("open");
  }
}
function restoreOpenStates() {
  for (var index in savedOpenState) {
    document.getElementById(index).setAttribute("open", savedOpenState[index]);
  }
}

function prefbarWriteToDisk() {
  gMainDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}

function prefbarGetTreeSelections(tree) {
  var selections = [];
  var select = tree.view.selection;
  if (select) {
    var count = select.getRangeCount();
    var min = new Object();
    var max = new Object();
    for (var i = 0; i < count; i++) {
      select.getRangeAt(i, min, max);
      for (var k=min.value; k<=max.value; k++) {
        if (k != -1) {
          selections[selections.length] = tree.contentView.getItemAtIndex(k).getAttribute("id");
        }
      }
    }
  }
  return selections;
}
